//
// This file is part of an OMNeT++/OMNEST simulation example.
//
// Copyright (C) 2010 OpenSim Ltd.
//
// This file is distributed WITHOUT ANY WARRANTY. See the file
// `license' for details on this and other legal matters.
//


#include <omnetpp.h>
#include "KmlHttpServer.h"
#include "KmlUtil.h"
#include "ChannelController.h"
#include "string.h"
#include "NodeInfo.h"
/**
 * A mobile node.
 */
const int maxnode=20;
int AMobileNode[maxnode][4],BMobileNode[maxnode][4],CMobileNode[maxnode][4],DMobileNode[maxnode][4];
class MobileNode : public cSimpleModule, public IKmlFragmentProvider, public IMobileNode
{
  protected:
    // configuration
    double playgroundLat,playgroundLon;  // NW corner of playground, in degrees
    double playgroundHeight,playgroundWidth;  // in meters
    double timeStep;
    unsigned int trailLength;
    std::string color;
    std::string modelURL;
    double modelScale;
    bool showTxRange;
    double txRange;
    double speed;


    int noofhostsinA,noofhostsinB,noofhostsinC,noofhostsinD;
    int AMobileid[];
    std::vector<NodeInfo> nodeList;
//    NodeInfo node[20];


    std::string ids[][1];

    // node position and heading (speed is constant in this model)
    double heading; // in degrees
    double x, y; // in meters, relative to playground origin

    std::vector<KmlUtil::Pt2D> path; // for visualization

    cModule *Node;
    bool isClusterHead;

    // utility function: converts playground-relative y coordinate (meters) to latitude
    double y2lat(double y) { return KmlUtil::y2lat(playgroundLat, y); }

    // utility function: converts playground-relative x coordinate (meters) to longitude
    double x2lon(double x) { return KmlUtil::x2lon(playgroundLat, playgroundLon, x); }

  public:
     MobileNode();
     virtual ~MobileNode();

     double getX() { return x; }
     double getY() { return y; }
     double getTxRange() { return txRange; }

  protected:
    virtual void initialize();
    virtual void handleMessage(cMessage *msg);
    virtual std::string getKmlFragment();
    virtual void findClusterHead();
//    virtual int Instr(std::string string1,std::string string2);
};

Define_Module(MobileNode);

MobileNode::MobileNode()
{

}

MobileNode::~MobileNode()
{
}
//virtual int Instr(std::string string1,std::string string2){
////    string1.at()
//}
void MobileNode::initialize()

{
//        Node = simulation.getModuleByPath("GoogleEarthNet");
//        Node=Node->nextp;
//        ev<<Node->getName();
    noofhostsinA=simulation.getSystemModule()->par("noofhostsinA");
    for(int i=0;i<maxnode;i++){
        AMobileNode[i][0]=0;
        AMobileNode[i][1]=0;
        AMobileNode[i][2]=0;
        AMobileNode[i][3]=0;
        BMobileNode[i][0]=0;
        BMobileNode[i][1]=0;
        BMobileNode[i][2]=0;
        BMobileNode[i][3]=0;
        CMobileNode[i][0]=0;
        CMobileNode[i][1]=0;
        CMobileNode[i][2]=0;
        CMobileNode[i][3]=0;
        DMobileNode[i][0]=0;
        DMobileNode[i][1]=0;
        DMobileNode[i][2]=0;
        DMobileNode[i][3]=0;
    }

//    ev<< "Hello World" << simulation.getSystemModule()->getNumParams();
    x = par("startX");
    y = par("startY");
    AMobileid[0]=1;
    heading = 360*dblrand();
    timeStep = par("timeStep");
    isClusterHead=false;
    getDisplayString().setTagArg("p", 0, x);
    getDisplayString().setTagArg("p", 1, y);

    playgroundLat = simulation.getSystemModule()->par("playgroundLatitude");
    playgroundLon = simulation.getSystemModule()->par("playgroundLongitude");
    playgroundHeight = simulation.getSystemModule()->par("playgroundHeight");
    playgroundWidth = simulation.getSystemModule()->par("playgroundWidth");

    trailLength = par("trailLength");
    modelURL = par("modelURL").stringValue();
    modelScale = par("modelScale");
    showTxRange = par("showTxRange");
    speed = par("speed");
    txRange = par("txRange");

    color = par("color").stringValue();
    if (color.empty())
    {
        // pick a color with a random hue
        char buf[16];
        double red,green,blue;
        KmlUtil::hsbToRgb(dblrand(), 1.0, 1.0, red, green, blue);
        sprintf(buf, "%2.2x%2.2x%2.2x", int(blue*255), int(green*255), int(red*255));
        color = buf;
    }

    KmlHttpServer::getInstance()->addKmlFragmentProvider(this);
    ChannelController::getInstance()->addMobileNode(this);

    if(strcmp(getName(),"AMobile")==0){
         AMobileNode[getIndex()][0]=x;
         AMobileNode[getIndex()][1]=y;
         AMobileNode[getIndex()][2]=getId();
 //        ev<<getName()<<"["<<getIndex()<<"]="<<AMobileNode[getIndex()][0]<<","<<AMobileNode[getIndex()][1]<<","<<AMobileNode[getIndex()][2];
     }
     if(strcmp(getName(),"BMobile")==0){
             BMobileNode[getIndex()][0]=x;
             BMobileNode[getIndex()][1]=y;
             BMobileNode[getIndex()][2]=getId();
     //        ev<<getName()<<"["<<getIndex()<<"]="<<AMobileNode[getIndex()][0]<<","<<AMobileNode[getIndex()][1]<<","<<AMobileNode[getIndex()][2];
         }
     if(strcmp(getName(),"CMobile")==0){
                 CMobileNode[getIndex()][0]=x;
                 CMobileNode[getIndex()][1]=y;
                 CMobileNode[getIndex()][2]=getId();
         //        ev<<getName()<<"["<<getIndex()<<"]="<<AMobileNode[getIndex()][0]<<","<<AMobileNode[getIndex()][1]<<","<<AMobileNode[getIndex()][2];
             }
     if(strcmp(getName(),"DMobile")==0){
                 DMobileNode[getIndex()][0]=x;
                 DMobileNode[getIndex()][1]=y;
                 DMobileNode[getIndex()][2]=getId();
         //        ev<<getName()<<"["<<getIndex()<<"]="<<AMobileNode[getIndex()][0]<<","<<AMobileNode[getIndex()][1]<<","<<AMobileNode[getIndex()][2];
             }
    // schedule first move
    cMessage *timer = new cMessage("move");
    scheduleAt(simTime(), timer);

}
void MobileNode::findClusterHead(){
    cModule *Node;
    int max=0;
    int index=-1;
    ev << "Calculating Cluster Head";
    if(strcmp(getName(),"AMobile")==0){
        for(int i=0;i<maxnode;i++){
            ev<<i<<"-"<<AMobileNode[i][2]<<"\n";
            if(AMobileNode[i][2]>max){
                max=AMobileNode[i][2];
                index=i;
            }
        }
    }
    ev<<"AMobile["<<index<<"]"<<"is the cluster head"<< "with ID"<<max;

//    Node = simulation.getModuleByPath(name);

//    int size=par("noofhostsinA");
//    if (!Node) error("server not found");
}
void MobileNode::handleMessage(cMessage *msg)
{
//    if(strcmp(getName(),"AMobile")==0){
//        findClusterHead();
//    }


    // update speed vector
    heading += 120*(dblrand()-0.5) * timeStep;
    heading = fmod(heading + 360, 360);

    // update position
    double vx = sin(heading*M_PI/180) * speed;
    double vy = -cos(heading*M_PI/180) * speed;
    x += vx*timeStep;
    y += vy*timeStep;

    // keep the node inside the playground
    if (x < 0) {x=0; heading = 360 - heading; }
    if (x > playgroundWidth) {x=playgroundWidth; heading = 360-heading;}
    if (y < 0) {y=0; heading = 180 - heading;}
    if (y > playgroundHeight) {y=playgroundHeight; heading = 180 - heading;}

    // store the position to be able to create a trail
    if (trailLength > 0)
        path.push_back(KmlUtil::Pt2D(x2lon(x),y2lat(y)));

    // Trail is at max length. Remove the oldest point to keep it at "trailLength"
    // note: this is not very efficient because entire vector is shifted down; should use circular buffer
    if (path.size () > trailLength)
        path.erase(path.begin());

    getDisplayString().setTagArg("p", 0, x);
    getDisplayString().setTagArg("p", 1, y);

    if (strcmp("Server1",getName()) == 0){
        ev << "Sending Data from Server1 to AMobileStaticNode";
        bubble("Sending Data from Server1 to AMobileStaticNode");
        Node = simulation.getModuleByPath("GoogleEarthNet.AMobileStaticNode");
        if (!Node) error("server not found");
        sendDirect(new cMessage("DATA1",0),Node->gate("in"));
    }
    if (strcmp("Server2",getName()) == 0){
        ev << "Sending Data from Server2 to BMobileStaticNode";
        bubble("Sending Data from Server2 to BMobileStaticNode");
        Node = simulation.getModuleByPath("GoogleEarthNet.BMobileStaticNode");
        if (!Node) error("server not found");
        sendDirect(new cMessage("DATA2",0),Node->gate("in"));
    }
    if (strcmp("Server3",getName()) == 0){
        ev << "Sending Data from Server3 to CMobileStaticNode";
        bubble("Sending Data from Server3 to CMobileStaticNode");
        Node = simulation.getModuleByPath("GoogleEarthNet.CMobileStaticNode");
        if (!Node) error("server not found");
        sendDirect(new cMessage("DATA3",0),Node->gate("in"));
    }
    if (strcmp("Server4",getName()) == 0){
        ev << "Sending Data from Server4 to DMobileStaticNode";
        bubble("Sending Data from Server4 to DMobileStaticNode");
        Node = simulation.getModuleByPath("GoogleEarthNet.DMobileStaticNode");
        if (!Node) error("Node not found");
        sendDirect(new cMessage("DATA4",0),Node->gate("in"));
    }
    if ((strcmp("DMobileStaticNode",getName()) == 0) and (strcmp(msg->getName(),"DATA4")==0)){
            ev << "Sending Data from DMobileStaticNode to Clusterhead";
            bubble("Sending Data from DMobileStaticNode to Clusterhead");
            Node = simulation.getModuleByPath("GoogleEarthNet.AMobile[0]");
            if (!Node) error("Node not found");
            sendDirect(new cMessage("DATA4",0),Node->gate("in"));
        }
    if ((strcmp("CMobileStaticNode",getName()) == 0) and (strcmp(msg->getName(),"DATA3")==0)){
                ev << "Sending Data from CMobileStaticNode to Clusterhead";
                bubble("Sending Data from CMobileStaticNode to Clusterhead");
                Node = simulation.getModuleByPath("GoogleEarthNet.CMobile[0]");
                if (!Node) error("Node not found");
                sendDirect(new cMessage("DATA3",0),Node->gate("in"));
            }
    if ((strcmp("BMobileStaticNode",getName()) == 0) and (strcmp(msg->getName(),"DATA2")==0)){
                    ev << "Sending Data from BMobileStaticNode to Clusterhead";
                    bubble("Sending Data from BMobileStaticNode to Clusterhead");
                    Node = simulation.getModuleByPath("GoogleEarthNet.BMobile[0]");
                    if (!Node) error("Node not found");
                    sendDirect(new cMessage("DATA2",0),Node->gate("in"));
                }
    if ((strcmp("AMobileStaticNode",getName()) == 0) and (strcmp(msg->getName(),"DATA1")==0)){
                        ev << "Sending Data from AMobileStaticNode to Clusterhead";
                        bubble("Sending Data from AMobileStaticNode to Clusterhead");
                        Node = simulation.getModuleByPath("GoogleEarthNet.AMobile[0]");
                        if (!Node) error("Node not found");
                        sendDirect(new cMessage("DATA1",0),Node->gate("in"));
                    }

    // schedule next move
    scheduleAt(simTime()+timeStep, msg);
}

std::string MobileNode::getKmlFragment()
{
    double longitude = x2lon(x);
    double latitude = y2lat(y);
    char buf[16];
    sprintf(buf, "%d", getIndex());
    std::string fragment;
    fragment += KmlUtil::folderHeader((std::string("folder_")+buf).c_str(), getFullName());

#ifdef USE_TRACK
    fragment += KmlUtil::track((std::string("track_")+buf).c_str(), path, timeStep, modelScale, modelURL.c_str(), "movement trail", NULL, (std::string("ff")+color).c_str());
#else
    fragment += KmlUtil::placemark((std::string("placemark_")+buf).c_str(), longitude, latitude, 2*modelScale, getFullName(), NULL);
    if (trailLength > 0)
        fragment += KmlUtil::lineString((std::string("trail_")+buf).c_str(), path, "movement trail", NULL, (std::string("ff")+color).c_str());
    if (showTxRange)
        fragment += KmlUtil::disk((std::string("disk_")+buf).c_str(), longitude, latitude, txRange, "transmission range", NULL, (std::string("40")+color).c_str());
    if (!modelURL.empty()) {
        double modelheading = fmod((360 + 90 + heading), 360);
        fragment += KmlUtil::model((std::string("model_")+buf).c_str(), longitude, latitude, modelheading, modelScale, modelURL.c_str(), "3D model", NULL);
    }
#endif

    fragment += "</Folder>\n";
    return fragment;
}

